Package: medicalcoder
Title: A Unified and Longitudinally Aware Framework for ICD-Based
        Comorbidity Assessment
Version: 0.8.1
Authors@R: c(
    person(given = "Peter",   family = "DeWitt",    email = "peter.dewitt@cuanschutz.edu",      role = c("aut", "cre", "cov"), comment = c(ORCID = "0000-0002-6391-0795")),
    person(given = "Tell",    family = "Bennett",   email = "tell.bennett@cuanschutz.edu",      role = c("ctb"),               comment = c(ORCID = "0000-0003-1483-4236")),
    person(given = "Seth",    family = "Russell",   email = "seth.russell@cuanschutz.edu",      role = c("ctb"),               comment = c(ORCID = "0000-0002-2436-1367")),
    person(given = "Meg",     family = "Rebull",    email = "meg.rebull@cuanschutz.edu",        role = c("ctb"),               comment = c(ORCID = "0000-0003-0334-4223")),
    person(given = "Vincent", family = "Rubinetti", email = "vincent.rubinetti@cuanschutz.edu", role = c("cov"),               comment = c(ORCID = "0000-0002-4655-3773"))
    )
Description: Provides comorbidity classification algorithms such as the
  Pediatric Complex Chronic Conditions (PCCC), Charlson, and Elixhauser indices,
  supports longitudinal comorbidity flagging across encounters, and includes
  utilities for working with medical coding schemas such as the International
  Classification of Diseases (ICD).
Depends: R (>= 3.5.0)
License: BSD_3_clause + file LICENSE
Language: en-US
Encoding: UTF-8
URL: http://www.peteredewitt.com/medicalcoder/,
        https://github.com/dewittpe/medicalcoder/
BugReports: https://github.com/dewittpe/medicalcoder/issues
LazyData: true
Suggests: data.table, dplyr, kableExtra, knitr, R.utils, rmarkdown
VignetteBuilder: knitr
Config/Needs/website: pkgdown, comorbidity, icdcomorbid,
        multimorbidity, dplyr, odbc, DBI, RSQLite, pccc
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-05 01:25:31 UTC; dewittp
Author: Peter DeWitt [aut, cre, cov] (ORCID:
    <https://orcid.org/0000-0002-6391-0795>),
  Tell Bennett [ctb] (ORCID: <https://orcid.org/0000-0003-1483-4236>),
  Seth Russell [ctb] (ORCID: <https://orcid.org/0000-0002-2436-1367>),
  Meg Rebull [ctb] (ORCID: <https://orcid.org/0000-0003-0334-4223>),
  Vincent Rubinetti [cov] (ORCID:
    <https://orcid.org/0000-0002-4655-3773>)
Maintainer: Peter DeWitt <peter.dewitt@cuanschutz.edu>
Repository: CRAN
Date/Publication: 2026-05-05 05:00:03 UTC
Built: R 4.5.2; ; 2026-05-05 05:17:01 UTC; unix
