# Generated by roxygen2: do not edit by hand

S3method(comorbidities,data.frame)
S3method(print,medicalcoder_comorbidities)
S3method(print,medicalcoder_comorbidities_with_subconditions)
S3method(summary,medicalcoder_comorbidities)
S3method(summary,medicalcoder_comorbidities_with_subconditions)
export(comorbidities)
export(get_charlson_codes)
export(get_charlson_index_scores)
export(get_elixhauser_codes)
export(get_elixhauser_index_scores)
export(get_elixhauser_poa)
export(get_icd_codes)
export(get_pccc_codes)
export(get_pccc_conditions)
export(icd_compact_to_full)
export(is_icd)
export(lookup_icd_codes)
