% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{mdcr}
\alias{mdcr}
\title{Synthetic Data}
\format{
\code{mdcr} is a \code{data.frame} with 4 columns, Each row is for one ICD id.
\itemize{
\item \code{patid}: patient identifier, integer values
\item \code{icdv}: ICD version; integer values, 9 or 10
\item \code{dx}: indicator column for ICD diagnostic (1) or procedure (0) codes
\item \code{code}: ICD code; character values
}
}
\usage{
mdcr
}
\description{
Synthetic Data
}
\seealso{
Other datasets:
\code{\link{mdcr_longitudinal}}
}
\concept{datasets}
\keyword{datasets}
