% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{mdcr_longitudinal}
\alias{mdcr_longitudinal}
\title{Synthetic Longitudinal Data}
\format{
\code{mdcr_longitudinal} is a \code{data.frame} with 4 columns.  The codes are
expected to be treated as diagnostic codes. Warning: there are a few ICD-9
codes which could match to procedure codes.
\itemize{
\item \code{patid}: patient identifier, integer values
\item \code{date}: date the diagnostic code was recorded
\item \code{icdv}: ICD version 9 or 10, integer valued
\item \code{code}: ICD codes; character values
}
}
\usage{
mdcr_longitudinal
}
\description{
Synthetic Longitudinal Data
}
\seealso{
Other datasets:
\code{\link{mdcr}}
}
\concept{datasets}
\keyword{datasets}
